import 'package:chopper/chopper.dart';

class RequestPaginate {
  static const int DEFAULT_SIZE = 20;
  static const String PARAM_HEADER_ITEMS_TOTAL = "x-total-count";
  static const String DEFAULT_PARAM_PAGE = "page";
  static const String DEFAULT_PARAM_SIZE = "size";

  String pageParamName, sizeParamName;
  int page, size;

  RequestPaginate(
      {this.pageParamName = DEFAULT_PARAM_PAGE,
      this.page = 0,
      this.sizeParamName = DEFAULT_PARAM_SIZE,
      this.size = DEFAULT_SIZE});

  Map<String, dynamic> build() {
    Map<String, dynamic> params = {};
    if (page >= 0) params[pageParamName] = page;
    if (size >= 0) params[sizeParamName] = size;
    return params;
  }

  static RequestPaginate fromMap(
    Map<String, dynamic>? map, {
    String defaultPageParamName = DEFAULT_PARAM_PAGE,
    int defaultPage = 0,
    String defaultSizeParamName = DEFAULT_PARAM_SIZE,
    int defaultSize = DEFAULT_SIZE,
  }) {
    RequestPaginate requestPaginate = RequestPaginate(
      pageParamName: defaultPageParamName,
      page: defaultPage,
      sizeParamName: defaultSizeParamName,
      size: defaultSize,
    );
    if (map == null || map.isEmpty) return requestPaginate;
    return requestPaginate
      ..page = map[defaultPageParamName] ?? defaultPage
      ..size = map[defaultSizeParamName] ?? defaultSize;
  }

  static int getPageTotal(
      {Response? response,
      Map<String, dynamic>? headers,
      int? xTotalCount,
      required int size}) {
    try {
      if (response != null) {
        return getPageTotal(headers: response.headers, size: size);
      }
      if (headers != null) {
        return getPageTotal(
            xTotalCount: int.parse(headers[PARAM_HEADER_ITEMS_TOTAL]),
            size: size);
      }
      return (xTotalCount! / size).ceil();
    } catch (e) {
      print(e);
    }
    return -1;
  }
}
