import 'package:adeptarch_base/src/widgets/widgets.dart';
import 'package:adeptarch_base/src/filters/filters.dart';
import 'package:flutter/material.dart';

// ignore_for_file: must_be_immutable
class TextFilterView extends BaseFilterView<TextFilter> {
  TextInputType keyboardType;

  TextFilterView({
    FilterController<TextFilter>? filterController,
    String? hint,
    bool? allowSorting,
    this.keyboardType = TextInputType.text,
  }) : super(
          filterController: filterController,
          hint: hint,
          allowSorting: allowSorting,
        );

  @override
  State createState() => _TextFilterViewState();
}

class _TextFilterViewState extends State<TextFilterView> {
  bool hasText = false;
  late Color colorPrimary, colorTextTertiary;
  final textController = AdeptTextEditingController();
  String? lastText;

  @override
  void initState() {
    super.initState();
    init();
    listeners();
  }

  void init() {
    onFilterChanged();
    lastText = textController.text;
    hasText = controllerHasText;
    textController.addListener(onTextChanged);
  }

  void listeners() {
    widget.filterController.addListener(onFilterChanged);
    textController.addListener(onTextChanged);
  }

  void onTextChanged() {
    String value = textController.text;
    if (widget.filterController.value != null) {
      if (!TextUtils.isEmpty(value)) {
        widget.filterController.value?.contains(value);
      } else {
        widget.filterController.value?.resetValues();
      }
    }
    if (TextUtils.isEmpty(value) && !TextUtils.isEmpty(lastText)) {
      setState(() {});
    } else if (!TextUtils.isEmpty(value) && TextUtils.isEmpty(lastText)) {
      setState(() {});
    }
    lastText = value;
    bool controllerHasText = this.controllerHasText;
    if (controllerHasText != hasText) {
      hasText = controllerHasText;
      setState(() {});
    }
  }

  void onFilterChanged() {
    TextFilter? filter = widget.filterController.value;
    if (filter?.value != null) textController.text = filter!.value!;
  }

  void checkColors(context) {
    colorPrimary = Theme.of(context).primaryColor;
    colorTextTertiary = Colors.grey.shade600;
  }

  bool get controllerHasText => textController.text.isNotEmpty;

  @override
  Widget build(BuildContext context) {
    checkColors(context);
    return Row(
      children: <Widget>[
        Expanded(
          child: TextField(
              controller: textController,
              keyboardType: widget.keyboardType,
              textInputAction: TextInputAction.done,
              decoration: InputDecoration(
                contentPadding: EdgeInsets.zero,
                labelText: widget.hint,
                labelStyle: TextStyle(
                    color: !TextUtils.isEmpty(textController.text)
                        ? colorPrimary
                        : colorTextTertiary),
                errorText: widget.filterController.error,
                suffixIcon: !hasText
                    ? null
                    : IconButton(
                        icon: Icon(
                          Icons.clear,
                          color: colorPrimary,
                        ),
                        onPressed: () {
                          textController.clear();
                        },
                      ),
              )),
        ),
        Visibility(
          child: Container(
            margin: const EdgeInsets.only(top: 16),
            alignment: Alignment.bottomCenter,
            child: SortView(
              sortMode: widget.filterController.value?.sortMode,
              listener: (oldMode, newMode) {
                widget.filterController.value?.sortMode = newMode;
              },
            ),
          ),
          visible: widget.allowSorting,
        ),
      ],
    );
  }

  @override
  void dispose() {
    widget.filterController.removeListener(onFilterChanged);
    textController.removeListener(onTextChanged);
    super.dispose();
  }
}
